"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParseNIK = void 0;
const address_1 = require("./address");
const monthNames = [
    'Januari',
    'Februari',
    'Maret',
    'April',
    'Mei',
    'Juni',
    'Juli',
    'Agustus',
    'September',
    'Oktober',
    'November',
    'Desember'
];
class ParseNIK {
    constructor(nik, name) {
        this.name = name;
        if (typeof nik === 'number')
            nik = String(nik);
        this.NIK = nik;
        let { date, gender } = this.getDateAndGender();
        this.month = this.normalizeZero(this.getPartNIK('month'));
        this.year = this.normalizeYear();
        this.date = date;
        this.province = this.getArea('province');
        this.city = this.getArea('city');
        this.district = this.getArea('district');
        this.lahir = this.getBirthDay();
        this.gender = gender;
        this.uniqcode = this.getUniqCode();
    }
    getPartNIK(part) {
        const partNIK = {
            'province': [0, 2],
            'city': [0, 4],
            'district': [0, 6],
            'date': [6, 8],
            'month': [8, 10],
            'year': [10, 12],
            'uniqcode': [12, 16]
        };
        const NIK = this.NIK;
        const order = partNIK[part];
        return Number(NIK.substring(order[0], order[1]));
    }
    getArea(area) {
        const areas = { 'province': 'provinces', 'city': 'cities', 'district': 'districts' };
        return address_1.findById(areas[area], this.getPartNIK(area));
    }
    getCurrentYear() {
        return parseInt(new Date().getFullYear().toString().substr(-2));
    }
    getDateAndGender() {
        let date = this.getPartNIK('date');
        let gender = 'LAKI-LAKI';
        if (date > 40) {
            gender = 'PEREMPUAN';
            date -= 40;
        }
        return { date: this.normalizeZero(date), gender };
    }
    normalizeZero(number) {
        return number > 10 ? number.toString() : `0${number}`;
    }
    getBirthDay() {
        const month = Number(this.month) - 1;
        if (Number(this.date) > 31 || month > 11)
            return false;
        return `${this.date} ${monthNames[month]} ${this.year}`;
    }
    normalizeYear() {
        const nikYear = this.getPartNIK('year');
        const currentYear = this.getCurrentYear();
        return nikYear < currentYear
            ? `20${nikYear > 10 ? nikYear : '0' + nikYear.toString()}`
            : `19${nikYear > 10 ? nikYear : '0' + nikYear.toString()}`;
    }
    getUniqCode() {
        let code = this.getPartNIK('uniqcode').toString();
        if (code.length >= 4)
            return code;
        const length = 4 - code.length;
        for (let i = 0; i < length; i++) {
            code = 0 + code;
        }
        return code;
    }
    isValid() {
        return this.NIK.length === 16 && !!this.province && !!this.city && !!this.district && !!this.lahir;
    }
    parse() {
        return {
            NIK: this.NIK,
            province: this.province,
            city: this.city,
            district: this.district,
            birthday: this.lahir,
            gender: this.gender,
            unicode: this.uniqcode,
            name: this.name
        };
    }
}
exports.ParseNIK = ParseNIK;
